#!/usr/bin/env python

##########
# Imports
####
import sys
import os
from insights_client.client_config import parse_config_file, InsightsClient
from insights_client.constants import InsightsConstants as insights_constants
from insights_client.auto_config import try_auto_configuration
from constants import OCPCoordinatorConstants
from cron import OCPCoordinatorCron
from logger import OCPCoordinatorLogger
from parser import OCPCoordinatorParser
from config import OCPCoordinatorConfig
from utilities import OCPCoordinatorUtilities


##########################
# Main
# 0) Run as root
# 1) Get OCP Constants
# 2) Get OCP Configuration (COMING SOON!)
# 3) Configure Insights uploader
# 4) Parse options
# 5) Set up Logger
# 6) Set up Cron
# 7) Setup Utilities
# 8) Run AP Coordinator
####
def main():

    # Run Coordinator as root
    if os.geteuid() is not 0:
        sys.exit("Insights Coordinator for OpenShift must be run as root")

    # AP Specific constants
    ap_constants = OCPCoordinatorConstants()

    # AP Configuration
    ap_config = OCPCoordinatorConfig(ap_constants)

    # Setup the Coordinator Logger
    ap_logger = OCPCoordinatorLogger(ap_constants, ap_config)
    ap_config.setup_logger(ap_logger)

    # Configure Insights Uploader
    uploader_conf = parse_config_file(insights_constants.default_conf_file)
    InsightsClient.config = uploader_conf
    try_auto_configuration()
    if not uploader_conf:
        sys.exit("Error configuring Insights Client. Please ensure you have Insights installed and it is up to date.")

    # Parse options
    options_parser = OCPCoordinatorParser(ap_constants, ap_config)
    options = options_parser.parse_options()
    options.uploader_conf = uploader_conf
    ap_logger.set_verbosity(options.verbose)

    # Are we running the configuration utility?
    if options.configure:
        ap_config.run_config()
        sys.exit("OpenShift Insights Coordinator Configuration utility completed.")

    # group param is required
    if not ap_config.get_config_var('group'):
        sys.exit("A group name is required but none was specified.")

    # Schedule the Coordinator Cron Job
    cron = OCPCoordinatorCron(ap_constants, ap_config, ap_logger)
    # TODO: This needs to check for disable_cron and enable_cron
    # Both enable_cron and disable_cron cant both be flagged or passed in as options
    # If both are passed in then exist with the appropriate message
    if options.enable_cron and not cron.cron_exists():
        cron.setup_cron()

    # Import the AP Coordinator Module
    # platform_module = None
    try:
        import ocp as platform_module
    except ImportError:
        sys.exit("Error importing libraries. Please try re-installing the insights-coordinator-ocp RPM.")

    if not platform_module:
        sys.exit("Error importing libraries. Please try re-installing the insights-coordinator-ocp RPM.")

    # AP Utilities
    ap_utilities = OCPCoordinatorUtilities(ap_constants, ap_logger, uploader_conf, ap_config)
    ap_config.setup_utilities(ap_utilities)

    # Run the Coordinator
    platform_module.run(options, ap_constants, ap_utilities, ap_logger, uploader_conf, ap_config)


########
# Main
####
if __name__ == "__main__":
    main()
