#!/usr/bin/env python
import ConfigParser


class OCPCoordinatorConfig:
    # Containers
    constants = None
    config = None
    temp_config = None
    logger = None
    utilities = None

    # Constructor
    def __init__(self, ocp_constants):
        self.constants = ocp_constants

    # Setup the AP Utilities
    def setup_utilities(self, utilities):
        self.utilities = utilities

    # Setup the AP Logger
    def setup_logger(self, logger):
        self.logger = logger

    # Get configuration variables from the AP coordinator configuration file
    # This will favor a temporary configuration value over an actual configuration value
    # This allows for the ability to override configuration values during runtime and as options
    def get_config_var(self, config_var):
        config = self.get_config()
        temp_config = self.get_temp_config()
        config_value = None
        temp_config_value = None
        try:
            config_value = config.get('insights-coordinator-ocp', config_var)
        except(ConfigParser.NoSectionError, ConfigParser.NoOptionError, ValueError, AttributeError):
            pass
        try:
            temp_config_value = temp_config.get('insights-coordinator-ocp', config_var)
        except(ConfigParser.NoSectionError, ConfigParser.NoOptionError, ValueError, AttributeError):
            pass
        if config_value or temp_config_value:
            if temp_config_value != config_value:
                return temp_config_value
            return config_value
        else:
            if not hasattr(self.constants, config_var):
                return None
            return getattr(self.constants, config_var)

    # Get the entire configuration object
    def get_config(self):
        if not self.config:
            self.setup_config()
        return self.config

    # Get a temporary configuration
    # This is a copy of the current configuration but allows
    # the user to override configuration values for this one execution
    # this allows an easier method to pass in temporary
    # options that will override existing configuration values
    def get_temp_config(self):
        if not self.temp_config:
            self.setup_config()
        return self.temp_config

    # Setup the configuration object
    # This will read the configuration file and convert it
    def setup_config(self):
        self.config = ConfigParser.RawConfigParser()
        self.temp_config = ConfigParser.RawConfigParser()
        self.config.read(self.constants.conf_file)
        self.temp_config.read(self.constants.conf_file)
        if not self.config.has_section('insights-coordinator-ocp'):
            self.config.add_section('insights-coordinator-ocp')
        if not self.temp_config.has_section('insights-coordinator-ocp'):
            self.temp_config.add_section('insights-coordinator-ocp')

    # Write a single configuration variable to the configuration file
    def write_config_var(self, config_var, config_val):
        config = self.get_config()
        config.set('insights-coordinator-ocp', config_var, config_val)
        with open(self.constants.conf_file, 'wb') as configfile:
            config.write(configfile)
        configfile.close()

    # Write the entire configuration file
    # { key1:val1, key2:val2 }
    def write_config_file(self, configuration_values):
        config = self.get_config()
        for key, value in configuration_values.iteritems():
            config.set('insights-coordinator-ocp', key, value)
        with open(self.constants.conf_file, 'wb') as configfile:
            config.write(configfile)
        configfile.close()

    def set_temp_config_var(self, temp_config_var, temp_config_val):
        temp_config = self.get_temp_config()
        temp_config.set('insights-coordinator-ocp', temp_config_var, temp_config_val)

    # Run the configuration utility
    # This will prompt for various user input for configuration variables then
    # Will rewrite the configuration file
    def run_config(self):

        self.logger.vlog("Running the Insights Coordinator for Open Shift Container Platform (OCP) Configuration Utility...")

        # TODO: This stuff should be in a language file
        # TODO: Create language.py -> self.langage.get_language('api_url') == 'Please enter the URL/Host...'

        # Configuration container
        config_values = {}

        ############
        # Prompt User for Configuration information
        ####

        # group for cluster nodes
        config_values['group'] = raw_input(
            'Please enter a group name for this OCP cluster: ')

        # Cron Configuration
        ####
        # Would you like the cron to run daily?
        config_values['cron_symlink'] = "/etc/cron.daily/insights-coordinator-ocp"
        cron_string = "Would you like to enable the OCP Insights Coordinator to run daily? Yy/Nn: "
        yes_values = ['y', 'yes', 'yea', 'yeah', 'yep', 'mmmhmm']
        no_values = ['n', 'no', 'nope', 'naw', 'nah']
        enable_cron = raw_input(cron_string).lower().strip().rstrip()
        while (enable_cron not in yes_values) and (enable_cron not in no_values):
            enable_cron = raw_input(cron_string).lower().strip().rstrip()
        if enable_cron in yes_values:
            config_values['enable_cron'] = True
        else:
            config_values['enable_cron'] = False

        ##################
        # Default Values
        ####
        config_values['verbose'] = False
        config_values['log_backup_count'] = 3
        config_values['insights_upload'] = True
        config_values['archive_only'] = False

        # Write all user configuration values to the file
        self.logger.vlog(
            "Finished running the Insights OCP Coordinator Configuration Utility. Writing configuration to %s." % (
                self.constants.conf_file))
        self.write_config_file(config_values)
        self.setup_config()
