#!/usr/bin/env python
###############################
# ap.py Platform Module
####


###########
# Imports
####
import os
import time
import json

from insights_client.collection_rules import InsightsConfig
from insights_client.connection import InsightsConnection


############################
# Run the platform module
# Main execution function
# Called from the main coordinator
#####
def run(options, constants, utilities, logger, insights_uploader_conf, config):
    logger.vlog("Running OpenShift Insights Coordinator")

    start_time = time.time()

    # create config file for client
    conn = InsightsConnection()
    rules_config = InsightsConfig(conn)
    collection_rules = rules_config.get_collection_rules(raw=True)
    sig = rules_config.fetch_gpg()
    branch_info = conn.branch_info()
    client_config = {
        "sig": sig,
        "uploader.json": collection_rules,
        "branch_info": branch_info,
        "machine-id": "{{ hostvars[inventory_hostname]['machine_id'] }}",  # <- ansible will expand this while copying
        "metadata": {
            "product_code": "ocp",
            "role": "{{ 'master' if hostvars[inventory_hostname]['master'] else 'node'}}"
        }
    }

    client_config = json.dumps(client_config)

    # save new config
    try:
        os.makedirs("/var/tmp/redhat-insights")
    except OSError:
        pass

    insights_config_file_name = "/var/tmp/redhat-insights/client.conf"
    insights_config_file = open(insights_config_file_name, "wb")
    insights_config_file.write(client_config)
    insights_config_file.close()

    # run insights on all the nodes (let ansible do the heavy lifting)
    utilities.run_command('ansible-playbook -i /var/lib/redhat-insights/ocp-nodes /var/lib/redhat-insights/insights.yml')

    # Upload node archives to red hat
    options.upload = True
    options.cleanup = True
    if options.upload:
        thetime = time.time() - start_time

        # for each archive....
        upload_success = utilities.upload_archives('/var/tmp/redhat-insights/nodes', thetime)

        if upload_success and options.cleanup:
            utilities.clean_tmps()
    else:
        logger.vlog("Archive saved to %s." % (utilities.get_tmpgz()))


def gather_metadata(utilities):
    # get metadata from this AP master
    # metadata.json:
    # {
    #     "product": "OCP",
    #     "system_id": "3836dbf2-c736-4d0d-9280-c4d8896c6ae5", // <- this needs to identify the Open Shift deployment
    #     "systems": [
    #         {
    #             "display_name": "openshift-node-1",
    #             "links": [
    #                 {
    #                     "system_id": "4bb603ae-64cc-4ed6-a840-c3c09127d237"
    #                 }
    #             ],
    #             "product": "OCP",
    #             "system_id": "9d290158-7236-5e11-94d3-45b46d2d1c68",
    #             "type": "node"
    #         },
    #         {
    #             "display_name": "openshift-node-2",
    #             "links": [
    #                 {
    #                     "system_id": "4bb603ae-64cc-4ed6-a840-c3c09127d237"
    #                 }
    #             ],
    #             "product": "OCP",
    #             "system_id": "e79638b6-c462-5f73-8845-1f0af53e1d0e",
    #             "type": "node"
    #         },
    #         {
    #             "display_name": "openshfit-master",
    #             "links": [],
    #             "product": "OCP",
    #             "system_id": "4bb603ae-64cc-4ed6-a840-c3c09127d237",
    #             "type": "master"
    #         }
    #     ]
    # }

    # populate metadata roles...
    ocp_masters = utilities.do('get_ocp_masters').split()
    ocp_nodes = utilities.do('get_ocp_nodes').split()

    # get system ids
    system_ids = {}

    for node in ocp_nodes:
        fname = utilities.get_tmpdir() + "/machine-ids/" + node
        fp = open(fname)
        with fp:
            system_ids[node] = fp.read()
        os.remove(fname)

    master_system_id = system_ids[ocp_masters[0]]

    # metadata = {
    #     "coordinator_version": "1.0.18",
    #     "display_name": "Open Shift Test",
    #     "insights_version": "insights-client-1.0.11-0.el7.centos.noarch",
    #     "product": "OCP",
    #     "nova_client_api_version": "2.23.0",
    #     "rhel_version": "Red Hat Enterprise Linux Server release 7.2 (Maipo)",
    #     "rhosp_version": "2015.1.2",
    #     "system_id": "c89b5b1e-1fcd-474d-826f-b8568051fd8f",
    #     "systems": [],
    #     "networks": [],
    #     "total_networks": 0
    # }

    metadata = {
        "product": "OCP",
        "display_name": "Open Shift Test",
        "systems": [],
        "networks": [],
        "total_networks": 0
    }

    # use cluster_id from this node as OCP system_id
    with open("/etc/insights-client/cluster-id") as f:
        metadata["system_id"] = f.read()

    for node in ocp_nodes:
        system = {}
        system["blacklisted"] = "false"
        system["display_name"] = node
        system["product"] = "OCP"
        system["ip"] = "1.1.1.1"
        system["status"] = "active"
        try:
            system["system_id"] = system_ids[node]
        except KeyError:
            system["system_id"] = "<node system_id missing>"

        system['links'] = []

        if node not in ocp_masters:
            system["type"] = "node"
            # add link to master
            system["links"].append({"system_id": master_system_id, "type": "master"})
        else:
            system["type"] = "master"

        metadata["systems"].append(system)

    # save metadata file to tmp dir
    try:
        os.makedirs("/var/tmp/redhat-insights/output")
    except OSError:
        pass

    # delete machine-ids
    os.rmdir(utilities.get_tmpdir() + "/machine-ids")

    return metadata


############
# Main
#####
if __name__ == "__main__":
    import doctest
    doctest.testmod()
