#!/usr/bin/env python
import subprocess
import os
import shutil
import tempfile
import uuid
from datetime import datetime
from datetime import date
from insights_client import connection
from insights_client.connection import InsightsConnection
from insights_client.collection_rules import InsightsConfig
from commands import APCoordinatorCommands


class OCPCoordinatorUtilities:
    # Containers
    constants = None
    logger = None
    tmpdir = None
    uploader_conf = None
    commands = None
    config = None

    # Constructor
    def __init__(self, rhev_constants, rhev_logger, uploader_conf, rhev_config):
        self.constants = rhev_constants
        self.logger = rhev_logger
        self.uploader_conf = uploader_conf
        self.commands = APCoordinatorCommands()
        self.config = rhev_config

    #######################
    ##### SHEXEC #########
    #####################

    # Takes a raw command to run
    # TODO: Have this return string results, or boolean results
    def run_command(self, cmd, return_type="String"):
        output = None
        if cmd:
            try:
                output = subprocess.Popen([cmd], shell=True, stdout=subprocess.PIPE).communicate()[0]
            except subprocess.CalledProcessError as e:
                self.logger.vlog(e.output, "ERROR")
                raise e
        return output

    # Get a command from the APCoordinatorCommands instance
    def get_command(self, cmd):
        if not self.commands:
            self.commands = APCoordinatorCommands()
        if hasattr(self.commands, cmd):
            return getattr(self.commands, cmd)
        else:
            return None

    def do(self, item):
        return self.run_command(self.get_command(item))

    #########################
    ###### TEMPS ###########
    #######################

    # Setup the temporary directory to store the archives
    def setup_tmpdir(self):
        tdir = '/var/tmp/redhat-insights'
        self.tmpdir = tdir
        self.logger.vlog("Setting up temporary directory %s" % (tdir))
        return tdir

    # Return the current temporary working directory
    def get_tmpdir(self):
        if not self.tmpdir:
            self.setup_tmpdir()
        return self.tmpdir

        # Clean the archives

    def clean_tmps(self):
        self.logger.vlog("Cleaning temporary files...")
        if self.get_tmpdir():
            self.logger.vlog("Removing directory %s" % (self.get_tmpdir()))
            shutil.rmtree(self.get_tmpdir())

    #######################
    ####### GZS ##########
    #####################

    # Return the current temporary GZ

    def get_tmpgz(self):
        return "%s.tar.gz" % (self.get_tmpdir())

    # Remove an individual archive gz
    def remove_gz(self, archivename):
        self.logger.vlog("Removing file %s" % (archivename))
        os.remove(archivename)

        # Untar an individual archive gz

    def untar_gz(self, archivename):
        untar_command = self.get_command('untar_gz') % (self.get_tmpdir(), archivename)
        untar_execution = int(self.run_command(untar_command))
        if untar_execution == 0:
            self.logger.vlog("Successfully untarred %s in %s" % (archivename, self.get_tmpdir()))
            return True
        else:
            self.logger.vlog("There was a problem untarring %s in %s" % (archivename, self.get_tmpdir()))
            return False

    # Retar a GZ
    def retar_gz(self, foldername):
        retar_command = self.get_command('retar_gz') % (foldername, foldername, foldername)
        retar_execution = int(self.run_command(retar_command))
        if retar_execution == 0:
            self.logger.vlog("Sucessfully retarred %s" % foldername)
            return True
        else:
            self.logger.vlog("There was a problem retarring %s" % foldername)
            return False

    ###########################
    #### ARCHIVES ############
    #########################

    # Compress an individual archive using stdout from a hypervisor
    def write_archive(self, stdout, archive_prefix=None, untar=True, remove_file=True, retar=True):
        tmpdir = self.get_tmpdir()
        if not stdout or stdout == "":
            return False
        if not archive_prefix:
            archive_prefix = self.constants.default_archive_prefix
        tfile = tempfile.NamedTemporaryFile(dir=tmpdir, prefix=archive_prefix, delete=False)
        tfile.write(stdout)
        tfile.flush()
        tfile.close()
        shutil.move(tfile.name, tfile.name + '.tar')
        self.logger.vlog("Wrote archive to %s" % (tfile.name + '.tar'))

    # Write a file...
    def write_file(self, stdout, filename):
        tmpdir = self.get_tmpdir()
        if not stdout or stdout == "":
            return False
        f = open(os.path.join(tmpdir, filename), 'wb')
        f.write(stdout)
        f.close()

    # Read a file...
    def read_file(self, filename):
        tmpdir = self.get_tmpdir()
        f = open(os.path.join(tmpdir, filename), 'rb')
        contents = f.read()
        f.close()
        return contents

        # Compress archives to be sent to Red Hat

    def compress_archives(self):
        tmpdir = self.get_tmpdir()
        tmpgz = self.get_tmpgz()

        # untar archives received from insights stdout
        self.run_command(self.get_command('untar_everything') % (tmpdir))

        # remove archives received from insights stdout
        self.run_command(self.get_command('remove_all_tars') % (tmpdir))

        # tar all archive folders
        self.run_command(self.get_command('tar_all_dirs') % (tmpdir))

        # remove all archive folders
        self.run_command(self.get_command('remove_all_dirs') % (tmpdir))

        # GZ up the whole folder
        tar_command = self.get_command('compress_archives') % (tmpdir, tmpgz)
        execution = int(self.run_command(tar_command))
        if execution == 0:
            self.logger.vlog("Successfully compressed all archives to %s" % (tmpgz))
            return True
        else:
            self.logger.vlog("There was an error compressing the archives to %s" % (tmpgz))
            return False

            # Upload archives to Red Hat

    def upload_archives(self, output_root, thetime):
        conn = connection.InsightsConnection()

        try:
            systems = []
            success = True
            group = self.config.get_config_var('group')

            for node in os.listdir(output_root):
                # TODO: expand this properly so the extension isn't hard-coded
                path = os.path.join(output_root, node, 'output.tar.gz')
                resp = conn.upload_archive(path, thetime, cluster=node)
                if resp.status_code == 201:
                    self.logger.vlog("Successfully uploaded %s to Red Hat." % (path))
                    # add system to group list
                    systems.append({'system_id': node})
                else:
                    self.logger.vlog("There was an issue uploading %s to Red Hat." % (path))
                    self.logger.vlog("API response code: %s" % (resp.status_code))
                    try:
                        j = resp.json()
                        if j.get("internal_error", {}).get("reference"):
                            self.logger.vlog("Failure reference key: %s" % (str(j["internal_error"]["reference"])))
                    except ValueError:
                        pass
                    self.logger.vlog("Keeping archive saved at %s" % (path))
                    success = False

            # add systems to group
            conn.group_systems(group, systems)

            return success

        except:
            self.logger.vlog("Failed to connect to Red Hat!")
            self.logger.vlog("Keeping archive saved at %s" % (output_root))
            return False

    #################
    #### JSON ######
    ###############

    # serialize date/time stuff

    def serialize_default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%dT%H:%M:%SZ')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        else:
            return str(obj)

    ###################
    #### INSIGHTS ####
    #################

    # Get the uploader conf
    def get_uploader_conf(self):
        return self.uploader_conf

    # Get the insights uploader.json and sig config
    def get_insights_config(self):
        rules_config = InsightsConfig(self.get_uploader_conf(), InsightsConnection(self.get_uploader_conf()))
        collection_rules = rules_config.get_collection_rules(raw=True)
        sig = rules_config.fetch_gpg()
        coord_config = {"sig": sig, "uploader.json": collection_rules}
        return coord_config

    ########################
    #### CLUSTER STUFF ####
    ######################

    # Get the cluster ID
    def get_cluster_id(self):
        if not os.path.exists(self.constants.cluster_id_file):
            return self.generate_cluster_id()
        else:
            f = open(self.constants.cluster_id_file, 'r')
            return f.read()

    # Generate the cluster ID
    def generate_cluster_id(self):
        f = open(self.constants.cluster_id_file, 'wb')
        cluster_id = uuid.uuid4().get_hex()
        f.write(cluster_id)
        f.close()
        return cluster_id
